#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "arrays.h"		//en esta biblioteca, estan las operaciones con arreglos
						//que utilizaremos. como crear arreglo aleatorio, o
						//copiar arreglo

int *ArrayA;
int *ArrayB;			//aca se crean los arreglos aleatorios

int *tempHeap;
int *tempQuick;			//aca se guardan los tiempos de cada iteracion para cada metodo.

int *frecArrayHeap;
int *frecArrayQuick;	//aca se guardan las veces cae un tiempo en un intervalo.

float *timeArrayHeap;
float *timeArrayQuick;	//aca va el tiempo medio de kaca intervalo.

int *MaxMin;			//en este arreglo se guardaran los tiempos maximos y minimos de kada metodo.

FILE *grafico;			//archivo de configuraciony ejecucion para el gnuplot.
FILE *tabla;			//archivo en donde se guardaran los datos a graficar.


/* **************************************************************** */
/*		Funcion que crea el archivo de configuracion de gnuplot 	*/
/*			aca se guarda la informacion sobre que archivo 			*/
/*				graficar y con que parametros.						*/
/* **************************************************************** */
void archivoGNUplot()
{
	char k = 34;
	grafico = fopen("grafico.gnu", "w");

	k = 34;
	fprintf(grafico,"set title %cHISTOGRAMA DE HEAPSORT Y QUICKSORT%c \n",k,k);
	fprintf(grafico,"set xlabel %cTiempo [us]%c\n",k,k);
	fprintf(grafico,"set ylabel %cFrecuencia de Ocurrencias [veces]%c\n",k,k);
	fprintf(grafico,"set autoscale\n\n");
	fprintf(grafico,"plot %ctabla1.dat%c using 1:2 title %cHeap Sort%c with histeps,",k,k,k,k);
	fprintf(grafico,"%ctabla2.dat%c using 1:2 title %cQuick Sort%c with histeps\n",k,k,k,k);
	fprintf(grafico,"pause -1\n");

	fclose(grafico);
	return;

	/* mas info sobre otras opciones, en:
		http://www.ucc.ie/gnuplot/gnuplot.html  */
}



/* **************************************************************** */
/* 		Funcion para insertar datos en un archivo de texto			*/
/* **************************************************************** */
void insertData(float A1[], int A2[], int intervalos, char tablaX[] )
{
	int i;
	tabla = fopen(tablaX,"w");
	for(i=0; i < intervalos; i++)
	{
		fprintf(tabla,"\t%f\t%d\n",A1[i],A2[i]);	//imprime los datos del arreglo A1 y A2
													//en el archivo tabla1 o tabla2 segun sea el caso
	}
	fclose(tabla);			// cierra el archivo despues de escribir en el.
	return;
}


/* **************************************************************** */
/*		Funcion para crear el arreglo con el tiempos medio			*/
/*			de cada intervalo.									*/
/* **************************************************************** */
void xArray(float frecArray[], int MIN, float delta, int intervalos)
{
	int j;
	for(j = 0; j < intervalos; j++)
	{
		frecArray[j] = (float)(MIN + delta*(j + 0.5));
	}
	return ;
}



/* **************************************************************** */
/*		Funcion Para clasificar los tiempos en un intervalo			*/
/* **************************************************************** */
float clasificar(int time[], int frecArray[], int MIN, int MAX, int intervalos, int repeticiones)
{
	float delta;
	int j, k;

	delta = (float)(MAX - MIN)/intervalos;		// calculamos el ancho de cada intervalo [us]
	for(j = 0; j < repeticiones; j++)
	{
		k = (int) ((time[j] - MIN)/delta);		//insertamos cada repeticion en un intervalo
		if(k == intervalos) k = k - 1 ;			//si time[i] = max entonces queda en el intervalo k+1

		frecArray[k] = frecArray[k]+1;			//incrementamos el intervalo en el que cayo la
												//repeticion i-esima
	}
	return delta;				//lo retornamos para un uso posterior.
}


/* **************************************************************** */
/*		Funcion para encontrar el valor MAYOR y MENOR del 			*/
/* 		arreglo "buscar" y los almacena en el arreglo "MaxMin"		*/
/* **************************************************************** */

int BuscarArray(int Buscar[], int MaxMin[], int repeticiones, int ini)
{
	int i;
	MaxMin[ini] = Buscar[0];
	for(i = 1; i < repeticiones; i++)
	{
		if(Buscar[i] < MaxMin[ini]) MaxMin[ini] = Buscar[i];
	}
	MaxMin[ini + 1] = Buscar[0];
	for(i = 1; i < repeticiones; i++)
	{
		if(Buscar[i] > MaxMin[ini + 1]) MaxMin[ini + 1] = Buscar[i];
	}
	return;
}


/* **************************************************************** */
/*			Funcion que asigna memoria para los arreglos			*/
/* **************************************************************** */

void AsignarMemoria(int largo, int repeticiones, int intervalos)
{

	ArrayA = (int *)malloc(largo*sizeof(int));
	ArrayB = (int *)malloc(largo*sizeof(int));
	tempHeap = (int *)malloc(repeticiones*sizeof(int));
	tempQuick = (int *)malloc(repeticiones*sizeof(int));
	timeArrayHeap = (float *)malloc(intervalos*sizeof(float));
	timeArrayQuick = (float *)malloc(intervalos*sizeof(float));
	frecArrayHeap = (int *)malloc(intervalos*sizeof(int));
	frecArrayQuick = (int *)malloc(intervalos*sizeof(int));
	MaxMin = (int *)malloc(4 * sizeof(int));
	return;
}


/* **************************************************************** */
/* 		funcion que libera la memoria de todos los arreglos.		*/
/* **************************************************************** */
void liberarMemoria()
{
	free(ArrayA);
	free(ArrayB);
	free(tempHeap);
	free(tempQuick);
	free(timeArrayHeap);
	free(timeArrayQuick);
	free(frecArrayHeap);
	free(frecArrayQuick);
	free(MaxMin);
}

/* **************************************************************** */
/*	 	Funcion Principal, encargada de llamar a QuickSort()		*/
/*				y HeapSort() entre otras...							*/
/* **************************************************************** */

int main(int argc, char *argv[])
{
	struct timeval TiempoOrdenar;
	int largo, repeticiones, intervalos;
	int semilla;
	int h;
	long ta, tb, tc;
	int quickMIN, quickMAX;
	int heapMIN, heapMAX;
	float deltaHeap, deltaQuick;

	// comprobamos si se ingresaron solo 2 argumentos para la funcion.
	if(argc != 3)
	{
		printf("Error de Parametros\a\a\n");
		printf("USO:  histograma  <nmero_de_repeticiones> <tamao_conjunto_datos> \n\n");
		return;
	}

	largo = strtol(argv[2], NULL, 10);
	repeticiones = strtol(argv[1], NULL, 10);

	//comprobamos que realmente se ingresaron numeros como argumentos.
	if(largo == 0 || repeticiones == 0 )
	{
		printf("Uno de los Parametros ingresados, no es vlido... \n");
		return;
	}

	//seteamos la semilla para el rand()
	gettimeofday(&TiempoOrdenar,NULL);
	semilla = TiempoOrdenar.tv_usec;
	srand(semilla);
	intervalos = (int)(1 + 3.3*log10(repeticiones));

	//asignamos memoria para todos los arreglos
	AsignarMemoria(largo, repeticiones, intervalos);

	//cada repeticion hacemos el mismo procedimiento
	//guardamos t1, ordenamos, guardamos t2, ordenamos, guardamos t3 y
	//despues hacemos las restas y las guardamos.
	for(h=0; h < repeticiones; h++)
	{
		MakeArray(ArrayA, largo);
		CopyArray(ArrayA, ArrayB, largo);

		gettimeofday(&TiempoOrdenar,NULL);
		ta = TiempoOrdenar.tv_usec + 1000000*TiempoOrdenar.tv_sec;
		heapSort(ArrayA, largo);

		gettimeofday(&TiempoOrdenar,NULL);
		tb = TiempoOrdenar.tv_usec + 1000000*TiempoOrdenar.tv_sec;
		quickSort(ArrayB, 0, largo-1);

		gettimeofday(&TiempoOrdenar,NULL);
		tc = TiempoOrdenar.tv_usec + 1000000*TiempoOrdenar.tv_sec;

		tempHeap[h] = tb - ta;
		tempQuick[h] = tc - tb;
	}

	//buscamos el menor y el mayor valor de los arreglos de tiempos.
	BuscarArray( tempHeap, MaxMin, repeticiones, 0);
	BuscarArray( tempQuick, MaxMin, repeticiones, 2);

	//informamos cual fue el mayor y menor tiempo para cada metodo, para tener una idea.
	printf("\nTiempos Maximos y Minimos: \n");
	printf("Heap[MIN, MAX] = [%d, %d]\n", MaxMin[0], MaxMin[1]);
	printf("Quick[MIN, MAX] = [%d, %d]\n", MaxMin[2], MaxMin[3]);

	//clasificamos el tiempo de cada repeticion en un intervalo.
	deltaHeap = clasificar(tempHeap, frecArrayHeap, MaxMin[0], MaxMin[1], intervalos, repeticiones);
	deltaQuick = clasificar(tempQuick, frecArrayQuick, MaxMin[2], MaxMin[3], intervalos, repeticiones);

	//imprimimos en pantalla el arreglo de intervalos, para tener una idea de lo que saldra
	//en el histagrama, esto se puede usar para comprobar si se esta graficando bien.
	printf("\nArreglo para el Histograma: \n");
	printf("HeapSort:\t");
	PrintArray(frecArrayHeap, intervalos);
	printf("QuickSort:\t");
	PrintArray(frecArrayQuick, intervalos);
	printf("\n");

	//hacemos nuestro arreglo para el eje x con los valores medios de
	//cada intervalo
	xArray(timeArrayHeap, MaxMin[0], deltaHeap, intervalos);
	xArray(timeArrayQuick, MaxMin[2], deltaQuick, intervalos);

	//insertamos los valores a graficar en la tabla:
	//eje x : valores medios de cada intervalo [us].
	//eje y : numero de veces que caen los tiempos en un determinado intervalo.
	//y los guardamos en archivos separados para cada metodo.
	insertData(timeArrayHeap, frecArrayHeap, intervalos, "tabla1.dat");
	insertData(timeArrayQuick, frecArrayQuick, intervalos, "tabla2.dat");

	//creamos el archivo de configuracion para el GNUplot
	archivoGNUplot();
	//ejecupamos el GNUplot con nuestro archivo como parametro.
	system("gnuplot grafico.gnu");

	//liberamos memoria... ^_^
	liberarMemoria();
	return;
}

